@@ -6,17 +6,19 @@ from django.http import HttpResponse
 
 class HttpCreated(HttpResponse):
     status_code = 201
-    
+
     def __init__(self, *args, **kwargs):
-        if 'location' in kwargs:
-            location = kwargs['location']
-            del(kwargs['location'])
-        
+        location = kwargs.pop('location', '')
+
         super(HttpCreated, self).__init__(*args, **kwargs)
         self['Location'] = location
 
 
 class HttpAccepted(HttpResponse):
+    status_code = 202
+
+
+class HttpNoContent(HttpResponse):
     status_code = 204
 
 
@@ -44,6 +46,10 @@ class HttpForbidden(HttpResponse):
     status_code = 403
 
 
+class HttpNotFound(HttpResponse):
+    status_code = 404
+
+
 class HttpMethodNotAllowed(HttpResponse):
     status_code = 405
 
@@ -56,6 +62,10 @@ class HttpGone(HttpResponse):
     status_code = 410
 
 
+class HttpTooManyRequests(HttpResponse):
+    status_code = 429
+
+
 class HttpApplicationError(HttpResponse):
     status_code = 500
 
