@@ -4,7 +4,10 @@ import django
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 from django.core.serializers import json
-from django.utils import simplejson
+try:
+    import json as simplejson
+except ImportError: # < Python 2.6
+    from django.utils import simplejson
 from django.utils.encoding import force_unicode
 from tastypie.bundle import Bundle
 from tastypie.exceptions import BadRequest, UnsupportedFormat
@@ -380,7 +383,7 @@ class Serializer(object):
         options = options or {}
 
         if lxml is None:
-            raise ImproperlyConfigured("Usage of the XML aspects requires lxml.")
+            raise ImproperlyConfigured("Usage of the XML aspects requires lxml and defusedxml.")
 
         return tostring(self.to_etree(data, options), xml_declaration=True, encoding='utf-8')
 
@@ -393,7 +396,7 @@ class Serializer(object):
         necessary.
         """
         if lxml is None:
-            raise ImproperlyConfigured("Usage of the XML aspects requires lxml.")
+            raise ImproperlyConfigured("Usage of the XML aspects requires lxml and defusedxml.")
 
         try:
             parsed = parse_xml(StringIO(content), forbid_dtd=forbid_dtd,
